@@ -1,6 +1,6 @@
 package Var::Pairs;
 
-our $VERSION = '0.001005';
+our $VERSION = '0.002001';
 
 use 5.014;
 use warnings;
@@ -181,8 +181,8 @@ sub kvs (+) {
 sub each_kv (+) {
     my ($container_ref) = @_;
 
-    # Uniquely identify this call, according to its lexical context...
-    my $ID = callsite();
+    # Uniquely identify this call, according to its lexical context and iteration target...
+    my $ID = callsite() . $container_ref;
 
     # Build an iterator...
     $iterator_for{$ID} //= [ &kvs ];
@@ -279,32 +279,35 @@ sub _get_each_kv {
 
 # Class implementing each key/value pair...
 package Var::Pairs::Pair {
-    use Hash::Util qw< fieldhash >;
     use Scalar::Util qw< looks_like_number >;
     use Data::Alias;
     use Carp;
 
     # Each pair object has two attributes...
-    fieldhash my %key_for;
-    fieldhash my %value_for;
+    my @key_for;
+    my @value_for;
+    my @freed;
 
     # Accessors for the attributes (value is read/write)...
-    sub value :lvalue { $value_for{shift()} }
-    sub index         {   $key_for{shift()} }
-    sub key           {   $key_for{shift()} }
+    sub value :lvalue { $value_for[${shift()}] }
+    sub index         {   $key_for[${shift()}] }
+    sub key           {   $key_for[${shift()}] }
+    sub kv            { my $self = shift;  $key_for[$$self], $value_for[$$self] }
 
     # The usual inside-out constructor...
     sub new {
         my ($class, $key, $container_ref, $container_type) = @_;
 
         # Create a scalar based object...
-        my $new_obj = bless \do{ my $scalar }, $class;
+        my $scalar = @key_for;
+        my $new_obj = bless \$scalar, $class;
 
         # Initialize its attributes (value needs to be an alias to the original)...
-        $key_for{$new_obj} = $key;
-        alias $value_for{$new_obj} = $container_type eq 'array' ? $container_ref->[$key]
-                                   : $container_type eq 'none'  ? $_[2]
-                                   :                              $container_ref->{$key};
+        $key_for[$scalar] = $key;
+        alias $value_for[$scalar] = $container_type eq 'array' ? $container_ref->[$key]
+                                  : $container_type eq 'none'  ? $_[2]
+                                  :                              $container_ref->{$key};
+        $freed[$scalar] = 0;
 
         return $new_obj;
     }
@@ -314,22 +317,40 @@ package Var::Pairs::Pair {
         # As a string, a pair is just: key => value
         q{""}   => sub {
             my $self = shift;
-            my $value = $value_for{$self};
+            my $value = $value_for[$$self];
             $value = ref $value                ? ref $value
                    : looks_like_number($value) ? $value
                    :                             qq{"$value"};
-            return "$key_for{$self} => $value";
+            return "$key_for[$$self] => $value";
         },
 
         # Can't numerify a pair (make it a hanging offence)...
         q{0+}   => sub { croak "Can't convert Pair(".shift.") to a number" },
 
         # All pairs are true (just as in Perl 6)...
-        q{bool} => sub { 1 },
+        q{bool} => sub { !!1 },
 
         # Everything else as normal...
         fallback => 1,
     );
+
+    sub DESTROY {
+        my $self = shift;
+
+        # Mark current storage as reclaimable...
+        $freed[$$self] = 1;
+
+        # Reclaim everything possible...
+        if ($freed[$#freed]) {
+            my $free_from = $#freed;
+            while ($free_from >= 0 && $freed[$free_from]) {
+                $free_from--;
+            }
+            splice @key_for,   $free_from+1;
+            splice @value_for, $free_from+1;
+            splice @freed,     $free_from+1;
+        }
+    }
 }
 
 1; # Magic true value required at end of module
@@ -343,7 +364,7 @@ Var::Pairs - OO iterators and pair constructors for variables
 
 =head1 VERSION
 
-This document describes Var::Pairs version 0.001005
+This document describes Var::Pairs version 0.002001
 
 
 =head1 SYNOPSIS
@@ -357,9 +378,9 @@ This document describes Var::Pairs version 0.001005
     }
 
 
-    # next_pair() iterates OO pairs from arrays and hashes...
+    # each_pair() iterates OO pairs from arrays and hashes...
 
-    while (my $next = next_pair %hash) {
+    while (my $next = each_pair %hash) {
         say $next->key, ' had the value ', $next->value;
         $next->value++;
     }
@@ -512,9 +533,11 @@ The typical scalar usage is:
     }
 
 In other words, C<each_kv()> is a drop-in replacement for Perl's
-built-in C<each()>, except that you can nest C<each_kv()>
-iterations over the same variable without shooting yourself in
-the foot.
+built-in C<each()>, with two exceptions: one an advantage, the other a
+limitation. The advantage is that you can nest C<each_kv()> iterations
+over the same variable without shooting yourself in the foot. The
+limitation is that, unlike C<each()>, C<each_kv()> does not reset
+when you call the C<keys> function on the hash you're iterating.
 
 =back
 
@@ -592,12 +615,32 @@ will increment each value in the C<%items> hash
 whose key starts with 'Q'.
 
 
+=item C<< $pair->kv >>
+
+Returns a two-element list containing copies of the key and the value of
+the pair. That is:
+
+    for my $item (pairs %items) {
+        my ($k, $v) = $item->kv;
+        say $v
+            if $k =~ /\d/;
+    }
+
+will print the value of every entry in the C<%items> hash
+whose key includes a digit.
+
+
 =item C<< "$pair" >>
 
 When used as a string, a pair is converted to a suitable representation
 for a pair, namely: C<< "I<key> => I<value>" >>
 
 
+=item C<< 0 + $pair >>
+
+Pairs cannot be used as numbers: an exception is thrown.
+
+
 =item C<< if ($pair) {...} >>
 
 When a pair is used as a boolean, it is always true.
@@ -827,6 +870,12 @@ Please report any bugs or feature requests to
 C<bug-var-pairs@rt.cpan.org>, or through the web interface at
 L<http://rt.cpan.org>.
 
+C<each_kv()> acts like a true one-time only iterator (in the OO sense),
+so there is no way to reset its iteration (i.e. the way that calling
+C<keys()> on a hash or array, resets any C<each()> that is iterating
+it). If you need to reset partially iterated hashes or arrays, you will
+need to use some other mechanism to do so.
+
 
 =head1 ACKNOWLEDGEMENTS
 
